# Hacking

This document list a couple of useful information to develop the NNS-dapp frontend.

## dapp development

NNS-dapp frontend uses an `.env` file to read various environment information and variables.

The repo itself does **not** contain any such file because the source of truth we are using is `dfx.json`.
That is why we are providing a `./config.sh` script that generate the above environment file automatically.

## Local

To run the dapp against canisters deployed locally on a simulated IC network, proceed as following:

- Deploy the Nns backend canisters locally with `dfx nns install`
- Once deployed, the canister IDs should be collected and the "local" network of `canister_ids.json` (**TODO**) should be updated accordingly manually
- Run `DFX_NETWORK=local ./config.sh` to populate the `.env` file (**TODO**)
- Start `npm run dev` in the `./frontend/` folder to serve the application

**TODO**

- we need to provide a sample of `canister_ids.json` in the repo or automate the task with a script because currently without it, the all setup isn't developer friendly
- fix `DFX_NETWORK=local ./config.sh` that does not output the canister ID in `VITE_OWN_CANISTER_ID` and `VITE_OWN_CANISTER_URL` of the `.env` file - i.e. makes unable to work locally

## Testnet

The [canister_ids.json] data provides the list of canister IDs available for various test environments.

### Configure

To develop and run locally the dapp against a testnet, proceed as following:

- Copy the [canister_ids.json] to the root of your local project
- Run `DFX_NETWORK=<testnet_name> ./config.sh` to populate the `.env` file
- Start `npm run dev` in the `./frontend/` folder to serve the application

e.g. replace `<testnet_name>` with `small11`

## e2e

e2e tests also need a `.env` configuration. Such file can also be generated with the help of the `./config.sh` script by providing a specific output parameter.

e.g. `DFX_NETWORK=<testnet_name> ENV_OUTPUT_FILE=./e2e-tests/.env ./config.sh`

### Running against local server

If you wish to run the e2e tests against your local server, configure the URL as following in `.env` file of the test:

```
VITE_OWN_CANISTER_URL=http://localhost:5173/
```

In addition, the II version deployed locally might be different than the version pinned to run the test. Therefore you might need to adapt the selectors in [./e2e-tests/components/ii-congratulations-page.ts](./e2e-tests/components/ii-congratulations-page.ts).

## Requirements

The `dfx` version installed locally should match the one defined in [dfx.json](https://github.com/dfinity/nns-dapp/blob/main/dfx.json). If not, you will have to either upgrade or manually change the version in the local file. In such case, please do not commit the change!

[canister_ids.json]: https://github.com/dfinity/nns-dapp/blob/testnets/testnets/canister_ids.json
[package.json]: https://github.com/dfinity/nns-dapp/blob/main/frontend/package.json

## ckBTC deployment

> Last update: 2023, Jan. 30

ckBTC deployment and development in NNS-dapp are in progress.

### Bitcoin network

To deploy a local Bitcoin network, the [documentation](https://internetcomputer.org/docs/current/developer-docs/integrations/bitcoin/local-development) on the IC website can be followed.

As ckBTC is not yet implemented in our pipeline, enabling it in `dfx.json` has not yet be done neither - i.e. it needs to be manually configured for testing / development purpose:

Add following in `defaults` of [dfx.json](./dfx.json).

```
"bitcoin": {
  "enabled": true,
  "nodes": [
    "127.0.0.1:18444"
  ]
}
```

### Ledger and minter

Deploying ckBTC ledger and minter are as well not yet automated.

To deploy these locally, a script is provided in [./scripts/ckbtc/deploy-ckbtc](./scripts/ckbtc/deploy-ckbtc).

However, this scripts requires downloading these canisters WASM and did files locally first and configuring these manually in [dfx.json](./dfx.json).

e.g. in `canisters`:

```
"ckbtc_test_minter": {
  "type": "custom",
  "candid": "tmp/minter.did",
  "wasm": "tmp/ckbtc_minter.wasm"
},
"ckbtc_test_ledger": {
  "type": "custom",
  "candid": "tmp/ledger.did",
  "wasm": "tmp/ledger_canister.wasm"
},
"ckbtc_test_index": {
  "type": "custom",
  "candid": "tmp/index.did",
  "wasm": "tmp/index_canister.wasm"
}
```

Once canisters deployed, their respective IDs shall be collected and updated in [canister-ids.constants.ts](./frontend/src/lib/constants/canister-ids.constants.ts).

Likewise, as the configuration is not yet automated, there are no `.env` variable that are yet generated.

### Index

The index canister does not exist yet on mainnet but, will be proposed soon. Same pattern will apply.

### Feature flag

Because the e2e tests are using the `local` environment to perform, we cannot enable the `ENABLE_CKBTC_LEDGER` per default neither. 

Therefore this flag should also be manually set to `true` in [dfx.json](./dfx.json) and the `.env` should be generated.